import os
#the name of your theme
BLOG_THEME_NAME = '4col'

#You probably don't want to manually edit this.  We're going to use convention to handle themeing
BLOG_THEME = {
    'name': BLOG_THEME_NAME,
    'static_path': 'blogger_themes/'+BLOG_THEME_NAME, #no leading or trailing /
    'template_path': 'themes/'+BLOG_THEME_NAME,
    # 'absolute_path': os.path.join(SITE_ROOT, 'Blogger/templates/themes/'+BLOG_THEME_NAME+'/')
}

BLOG_INFO = {
    'BLOG_TITLE': 'My Blog Name',
    'BLOG_SUBTITLE': 'Blog subname',
    'BLOG_THEME_DIR': BLOG_THEME['static_path']
}

BLOG_SETTINGS = {
    'auto_publish': True, #as you can see from the admin page, this doesn't have an effect
}


from django.db import models
from django.core.urlresolvers import reverse
from django.contrib.auth.models import User
from Blogger.settings import BLOG_SETTINGS
from django.db.models import Sum
from Blogger.managers import PostManager
from django.contrib.syndication.views import Feed
import unidecode
import re

class Tag(models.Model):
    """
    A string to be associated with another model
    """
    
    name = models.CharField(max_length=200, unique=True)
    slug = models.SlugField(max_length=200, unique=True)
    
    def __unicode__(self):
        return self.name

    def get_absolute_url(self):
        return reverse('tag_archive', args=[str(self.slug)])

    def number_of_uses(self):
        """Return count of post_set"""
        return self.post_set.count()
    number_of_uses.short_description = "Number of uses"


class Author(models.Model):
    """
    Association of a Blog Author data and a django user
    """

    first_name = models.CharField(max_length=200)
    last_name = models.CharField(max_length=200)
    user = models.ForeignKey(User)

    class Meta:
        unique_together = (("first_name", "last_name"))    

    def __unicode__(self):
        return ' '.join([self.first_name, self.last_name])

    def get_absolute_url(self):
        return reverse('author_archive', args=['-'.join([self.first_name, self.last_name])])

    def number_of_posts(self):
        """Return count of post_set"""
        return self.post_set.count()
    number_of_posts.short_description = "Number of posts"

class Post(models.Model):
    """
    User generated blog post
    """

    author = models.ForeignKey(Author, blank=True)
    title = models.CharField(max_length=200, unique=True)
    body = models.TextField()
    created_at = models.DateTimeField(auto_now_add=True)
    published = models.BooleanField(default=BLOG_SETTINGS['auto_publish']) #TODO: i don't think this is working.  would like to make it work.
    tags = models.ManyToManyField(Tag, blank=True)
    slug = models.SlugField(max_length=200, unique=True)
    
    objects = models.Manager() # The default manager.
    popular_posts = PostManager()

    class Meta:
        verbose_name_plural = "posts"

    def __unicode__(self):
        return self.title
    
    def get_absolute_url(self):
        return reverse('view_post', args=[str(self.slug)])

    def get_tags(self):
        """Returns tags as a composite string"""
        names = ', '.join([t.name for t in self.tags.all()])
        if len(names) > 20:
            names = names[:20] + "..."
        return names
    get_tags.short_description = "Tags"

    #slug field should remove the need for this.  Leaving it here for a while until I'm sure
    # def set_slug(self):
    #     """Sets self.slug from self.title"""
    #     title_str = unidecode.unidecode(self.title).lower()
    #     self.slug = re.sub(r'\W+','-',title_str)



from django.db import models
from django.contrib.contenttypes.models import ContentType



#http://stackoverflow.com/questions/8215570/ordering-entries-via-comment-count-with-django
#https://docs.djangoproject.com/en/dev/topics/db/managers/#custom-managers-and-model-inheritance
class PostManager(models.Manager):

	"""A custom manager for Post models"""
	entry_type = None
	def get_query_set(self):
		if not self.entry_type:
			self.entry_type = ContentType.objects.get(model='post')
		"""attaches django comment_count to all querysets"""
		return super(PostManager,self).get_query_set().all().extra(select={
                'comment_count': """SELECT COUNT(*) FROM django_comments
                    WHERE django_comments.object_pk = Blogger_post.id
                    AND django_comments.content_type_id = %s"""
            }, select_params=(self.entry_type.id,)).order_by('-comment_count')



from django.contrib.syndication.views import Feed
from django.core.urlresolvers import reverse
from Blogger.models import Post

class LatestEntriesFeed(Feed):
    title = "Blog RSS"
    link = "/rss/"
    description = "Updates on changes and additions to this blog."

    def items(self):
        return Post.objects.order_by('-created_at')[:5]

    def item_title(self, item):
        return item.title

    def item_description(self, item):
        return item.body

    # item_link is only needed if Post has no get_absolute_url method.
    def item_link(self, item):
        return reverse('view_post', args=[item.slug])



from django.contrib import admin
from Blogger.models import Tag, Post, Author

class TagAdmin(admin.ModelAdmin):
	list_display = ('name', 'number_of_uses')
	search_fields = ('name',)
	prepopulated_fields = {"slug": ("name",)}

class PostAdmin(admin.ModelAdmin):
	exclude = ('author',)
	date_hierarchy = 'created_at'
	list_display = ('title', 'author', 'created_at', 'published', 'get_tags')
	list_filter = ['author', 'created_at', 'published']
	search_fields = ('title', 'body')
	prepopulated_fields = {"slug": ("title",)}

	#http://stackoverflow.com/questions/753704/manipulating-data-in-djangos-admin-panel-on-save
	def save_model(self, request, obj, form, change):
		"""Customize save method via admin panel save"""
		if not change:
			obj.author = request.user.author_set.all()[0]
		#obj.set_slug()
		obj.save()


class AuthorAdmin(admin.ModelAdmin):
	list_display = ('first_name', 'last_name', 'user', 'number_of_posts')
	search_fields = ('first_name', 'last_name')

class CommentAdmin(admin.ModelAdmin):
	pass

admin.site.register(Tag, TagAdmin)
admin.site.register(Post,PostAdmin)
admin.site.register(Author, AuthorAdmin)



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding unique constraint on 'Author', fields ['first_name', 'last_name']
        db.create_unique('Blogger_author', ['first_name', 'last_name'])

        # Adding unique constraint on 'Post', fields ['title']
        db.create_unique('Blogger_post', ['title'])


        # Changing field 'Post.slug'
        db.alter_column('Blogger_post', 'slug', self.gf('django.db.models.fields.SlugField')(unique=True, max_length=200))
        # Adding index on 'Post', fields ['slug']
        db.create_index('Blogger_post', ['slug'])

        # Adding unique constraint on 'Post', fields ['slug']
        db.create_unique('Blogger_post', ['slug'])

        # Adding unique constraint on 'Tag', fields ['name']
        db.create_unique('Blogger_tag', ['name'])


    def backwards(self, orm):
        # Removing unique constraint on 'Tag', fields ['name']
        db.delete_unique('Blogger_tag', ['name'])

        # Removing unique constraint on 'Post', fields ['slug']
        db.delete_unique('Blogger_post', ['slug'])

        # Removing index on 'Post', fields ['slug']
        db.delete_index('Blogger_post', ['slug'])

        # Removing unique constraint on 'Post', fields ['title']
        db.delete_unique('Blogger_post', ['title'])

        # Removing unique constraint on 'Author', fields ['first_name', 'last_name']
        db.delete_unique('Blogger_author', ['first_name', 'last_name'])


        # Changing field 'Post.slug'
        db.alter_column('Blogger_post', 'slug', self.gf('django.db.models.fields.CharField')(max_length=200))

    models = {
        'Blogger.author': {
            'Meta': {'unique_together': "(('first_name', 'last_name'),)", 'object_name': 'Author'},
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '200'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '200'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"})
        },
        'Blogger.post': {
            'Meta': {'object_name': 'Post'},
            'author': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['Blogger.Author']", 'blank': 'True'}),
            'body': ('django.db.models.fields.TextField', [], {}),
            'created_at': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'published': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '200', 'blank': 'True'}),
            'tags': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['Blogger.Tag']", 'symmetrical': 'False', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '200'})
        },
        'Blogger.tag': {
            'Meta': {'object_name': 'Tag'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '200'})
        },
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        }
    }

    complete_apps = ['Blogger']


# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Tag'
        db.create_table('Blogger_tag', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=200)),
        ))
        db.send_create_signal('Blogger', ['Tag'])

        # Adding model 'Author'
        db.create_table('Blogger_author', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=200)),
            ('user', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User'])),
        ))
        db.send_create_signal('Blogger', ['Author'])

        # Adding model 'Post'
        db.create_table('Blogger_post', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('author', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['Blogger.Author'], blank=True)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=200)),
            ('body', self.gf('django.db.models.fields.TextField')()),
            ('created_at', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
        ))
        db.send_create_signal('Blogger', ['Post'])

        # Adding M2M table for field tags on 'Post'
        db.create_table('Blogger_post_tags', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('post', models.ForeignKey(orm['Blogger.post'], null=False)),
            ('tag', models.ForeignKey(orm['Blogger.tag'], null=False))
        ))
        db.create_unique('Blogger_post_tags', ['post_id', 'tag_id'])

        # Adding model 'Comment'
        db.create_table('Blogger_comment', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('creator', self.gf('django.db.models.fields.CharField')(max_length=200)),
            ('email', self.gf('django.db.models.fields.EmailField')(max_length=75)),
            ('created_at', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('body', self.gf('django.db.models.fields.TextField')()),
            ('post', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['Blogger.Post'])),
        ))
        db.send_create_signal('Blogger', ['Comment'])


    def backwards(self, orm):
        # Deleting model 'Tag'
        db.delete_table('Blogger_tag')

        # Deleting model 'Author'
        db.delete_table('Blogger_author')

        # Deleting model 'Post'
        db.delete_table('Blogger_post')

        # Removing M2M table for field tags on 'Post'
        db.delete_table('Blogger_post_tags')

        # Deleting model 'Comment'
        db.delete_table('Blogger_comment')


    models = {
        'Blogger.author': {
            'Meta': {'object_name': 'Author'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '200'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"})
        },
        'Blogger.comment': {
            'Meta': {'object_name': 'Comment'},
            'body': ('django.db.models.fields.TextField', [], {}),
            'created_at': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'creator': ('django.db.models.fields.CharField', [], {'max_length': '200'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'post': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['Blogger.Post']"})
        },
        'Blogger.post': {
            'Meta': {'object_name': 'Post'},
            'author': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['Blogger.Author']", 'blank': 'True'}),
            'body': ('django.db.models.fields.TextField', [], {}),
            'created_at': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'tags': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['Blogger.Tag']", 'symmetrical': 'False', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '200'})
        },
        'Blogger.tag': {
            'Meta': {'object_name': 'Tag'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '200'})
        },
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        }
    }

    complete_apps = ['Blogger']


from django.conf.urls import patterns, include, url
from Blogger.feeds import LatestEntriesFeed

urlpatterns = patterns('Blogger.views',
	url(r'^$', 'list', name='all_archive'),
    url(r'^(?P<year>\d{4})/$', 'list',name="yearly_archive"),
    url(r'^(?P<year>\d{4})/(?P<month>\d{2})/$', 'list', name="monthly_archive"),
    url(r'^tag/(?P<tag>[\w-]+)/$', 'list', name='tag_archive'),
    url(r'^author/(?P<author>[\w-]+)/$', 'list', name='author_archive'),
    url(r'^confirm/comment/$', 'comment_posted', name='comment_posted'),
    url(r'^post/(?P<slug>[\w-]+)/$', 'view_post', name='view_post'),
    (r'^rss/$', LatestEntriesFeed()),
)	


from django.http import HttpRequest, HttpResponse
from django.shortcuts import render_to_response, redirect
from django.core.urlresolvers import reverse
from django.template import RequestContext
from Blogger.models import Post, Tag, Author
from django.db.models import Sum
from django.contrib import messages
from django.contrib.comments import Comment
import datetime

from Blogger import settings as blogger_settings

def get_sidebar_data():
    popular_posts = Post.popular_posts.all()[:5] #use this for the proper way when implemented
    recent_posts = Post.objects.filter(published=True).order_by('-created_at')[:5]
    archive = Post.objects.all().dates('created_at','month',order='DESC')
    tags = Tag.objects.all()
    authors = Author.objects.all()
    data = {
        'popular_posts': popular_posts,
        'recent_posts': recent_posts,
        'archive': archive,
        'tags': tags,
        'authors': authors,
    }
    return data

def render_on_list(request, data):
    #this is a way to do themeing
    #print blogger_settings.BLOG_THEME_NAME + '/list.html'
    return render_to_response('list.html', data, context_instance=RequestContext(request))
    #return render_to_response('themes/3col/list.html', data, context_instance=RequestContext(request))

def comment_posted(request):
    
    c = request.GET['c']
    messages.add_message(request, messages.SUCCESS, 'Commented added successfully.')
    c = Comment.objects.get(pk=c)
    return redirect(c.content_object)

# Create your views here.
def list(request, year=None, month=None, tag=None, author=None):
    
    sidebar_data = get_sidebar_data()
    
    data = {
            'posts': None,
            'section_title': 'Posts'
        }

    data.update(sidebar_data)

    if tag:
        posts = Post.objects.filter(tags__slug=tag)
        data['posts'] = posts
        data['section_title'] = "Posts"
        return render_on_list(request, data)
    if author:
        fname, lname = author.split('-')
        posts = Post.objects.filter(author__first_name=fname, author__last_name=lname)
        data['posts'] = posts
        data['section_title'] = "Posts"
        return render_on_list(request, data)
    if not year:
        posts = Post.objects.all().order_by('-created_at')
        data['posts'] = posts
        data['section_title'] = "Posts"
        return render_on_list(request, data)
        #Recent posts reverse chrono
    if not month:
        posts = Post.objects.filter(created_at__year=year).order_by('-created_at')
        data['posts'] = posts
        data['section_title'] = "Yearly Archive"
        return render_on_list(request, data)
    else:
        posts = Post.objects.filter(created_at__year=year).filter(created_at__month=month).order_by('-created_at')
        data['posts'] = posts
        data['section_title'] = "Monthly Archive"
        return render_on_list(request, data)
    

def view_post(request, slug):
    
    post = Post.objects.get(slug=slug)
    sidebar_data = get_sidebar_data()
    data = {
        'post': post,
    }
    data.update(sidebar_data)
    return render_to_response('view_post.html', data, context_instance=RequestContext(request))

def archive_time(request):
    pass

def archive_category(request, category=None):
    pass

def archive_author(request, author=None):
    pass


    


import os
#from distutils.core import setup
from setuptools import setup, find_packages

setup(
    name = "django-blogger",
    version = "0.2.0.8",
    author = "Jake Gaylor",
    author_email = "jake@codegur.us",
    description = "A package to administer a simple blog via Django.",
    url = "https://github.com/jhgaylor/django-simple-blog/",
    install_requires = [ln for ln in open('requirements.txt')],
    packages = find_packages(), 
    #package_data=package_data,
    include_package_data = True,
)